package download

import (
	"downloader/internal/app/api/common/responses"
	"downloader/internal/core/entities"
	"errors"
	"github.com/go-chi/render"
	"net/http"
)

type requestDownloadRequest struct {
	Provider string `json:"provider"`
	Link     string `json:"link"`
}

type requestDownloadResponse struct {
	*entities.Download
}

func (_ requestDownloadResponse) Render(_ http.ResponseWriter, _ *http.Request) error {
	return nil
}

func (dr *requestDownloadRequest) Bind(r *http.Request) error {
	if dr.Link == "" || dr.Provider == "" {
		return errors.New("missing required download request field")
	}

	return nil
}

func (a *api) requestDownload(w http.ResponseWriter, r *http.Request) {
	data := &requestDownloadRequest{}
	if err := render.Bind(r, data); err != nil {
		_ = render.Render(w, r, responses.ErrInvalidRequest(err))
		return
	}

	download, err := a.drService.Schedule(data.Provider, data.Link)
	if err != nil {
		_ = render.Render(w, r, responses.ErrInvalidRequest(err))
		return
	}

	render.Status(r, http.StatusAccepted)
	_ = render.Render(w, r, newRequestDownloadResponse(download))
}

func (a *api) getDownloadById(w http.ResponseWriter, r *http.Request) {
	downloadId := r.Context().Value("downloadId").(string)

	download, err := a.drService.Get(downloadId)
	if err != nil {
		_ = render.Render(w, r, responses.ErrNotFound())
		return
	}

	if err := render.Render(w, r, newRequestDownloadResponse(download)); err != nil {
		_ = render.Render(w, r, responses.ErrInvalidRequest(err))
		return
	}
}

func newRequestDownloadResponse(download *entities.Download) *requestDownloadResponse {
	return &requestDownloadResponse{Download: download}
}
